__author__ = 'ikibalin'
__version__ = "2020_09_10"

import os
import sys
 
from PyQt5 import QtWidgets, QtGui, QtCore

import numpy
import matplotlib
import matplotlib.backends.backend_qt5agg
import matplotlib.figure 
import matplotlib.pyplot

class cwind_central(QtWidgets.QMainWindow):
    def __init__(self):
        super(cwind_central, self).__init__()
        self.title = "program 'Graph'"
        self.setWindowTitle(self.title)
        widg_central = cwidg_central()
        self.setCentralWidget(widg_central)

         
        np_x = numpy.linspace(1, 2, 10)
        np_y = numpy.square(np_x)
        np_s = numpy.square(np_x)*0.2
        np_xm = numpy.vstack((np_x, np_y)).transpose()
        np_xes = numpy.vstack((np_x, np_y+1, np_s)).transpose()
        np_xesm = numpy.vstack((np_x, np_y+2, np_s, np_y+2.1)).transpose()
        widg_central.plot_numpy_arrays(np_x, np_xm, np_xes, np_xesm)
         
        self.show()

class cwidg_central(QtWidgets.QWidget):
    def __init__(self):
        super(cwidg_central, self).__init__()
        self.init_layout_central()
        self.setLayout(self.layout_central)
 
    def init_layout_central(self):
        lay_main = QtWidgets.QHBoxLayout()

        self.graph = Graph(self, width=5, height=4)
        """
        lay_1 = QtWidgets.QVBoxLayout()
        _b_gm = QtWidgets.QPushButton("values to clipboard")
        _b_gm.clicked.connect(self.give_values)
        lay_1.addStretch(1)
        lay_1.addWidget(_b_gm)
        """
        lay_main.addWidget(self.graph)
        #lay_main.addLayout(lay_1)

        self.layout_central = lay_main
        
    def plot_numpy_arrays(self, *argv):
        """
Keyword arguments:

    np_x: 2D array (:, 1) with columns: position of x points
    np_xm: 2D array (:, 2) with columns: x, model points
    np_xes: 2D array (:, 3) with columns: x, experimental points, sigma
    np_xesm: 2D array (:, 4) with columns: x, experimental points, sigma, model points

Example:

>>> self.plot_file(np_xes, np_xesm, np_xm)
>>> self.plot_file(np_xm)
>>> self.plot_file(np_xm, np_x)
        """
        self.graph.numpy_arrays = argv
        self.graph.set_data_to_graph()

    def give_values(self):
        """
Save information in clipboard
        """
        ls_out = ["Not realized."]
        cb = QtWidgets.QApplication.clipboard()
        cb.clear(mode=cb.Clipboard)
        cb.setText("\n".join(ls_out), mode=cb.Clipboard)


class Graph(matplotlib.backends.backend_qt5agg.FigureCanvasQTAgg):
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = matplotlib.figure.Figure(figsize=(width, height), dpi = dpi)
        fig.subplots_adjust(left = 0.07,
                            right = 0.97,
                            top = 0.97,
                            bottom = 0.07,
                            wspace = 0.0,
                            hspace = 0.0)
        
        super(Graph, self).__init__(fig)
        self.info_press = (False, False)
        self.xlim_orig = (0., 1.)
        self.ylim_orig_1 = (0., 1.)
        self.ylim_orig_2 = (-0.5, 0.5)
        self.numpy_arrays = []

        self.control = parent
        self.figure = fig
        
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)

        self.grid = matplotlib.figure.GridSpec(4, 4)
        self.ax_pri_1, self.ax_pri_2 = None, None

        fig.canvas.mpl_connect("button_press_event", self.onclick)
        
    def set_data_to_graph(self):


        flag_ax_1, flag_ax_2 = False, False
        for np_array in self.numpy_arrays:
            n12 = np_array.shape
            if len(n12) == 1:
                flag_ax_2 = True
            else:
                if (n12[1] in (2, 3)):
                    flag_ax_1 = True
                elif (n12[1] == 4):
                    flag_ax_1 = True
                    flag_ax_2 = True

        if (flag_ax_1 & flag_ax_2): 
            self.ax_pri_1 = self.figure.add_subplot(self.grid[:-1, :], xticklabels=[])
            self.ax_pri_1.cla()
            self.ax_pri_2 = self.figure.add_subplot(self.grid[-1, :], xticklabels=[])
            self.ax_pri_2.cla()
        elif flag_ax_1: 
            self.ax_pri_1 = self.figure.add_subplot(self.grid[:, :], xticklabels=[])
            self.ax_pri_1.cla()
        elif flag_ax_2: 
            self.ax_pri_2 = self.figure.add_subplot(self.grid[:, :], xticklabels=[])
            self.ax_pri_2.cla()

        col_0 = "#000000"
        col_1 = "#FF0000"
        col_2 = "#0000FF"
        col_3 = "#006400"
        x_lim_min, x_lim_max = None, None
        y_lim_min, y_lim_max = None, None
        y_lim_min_2, y_lim_max_2 = -0.5, 0.5

        if flag_ax_2: self.ax_pri_2.axhline(y=0., linewidth=0.5, color="k", linestyle="--")

        for np_array in self.numpy_arrays:

            n12 = np_array.shape
            if len(n12) == 2:
                n1, n2 = n12
                x = np_array[:, 0]
            elif len(n12) == 1:
                n1, n2 = n12[0], 1
                x = np_array[:]
            else:
                return

            if x_lim_min is not None:
                x_lim_min = min((numpy.nanmin(x), x_lim_min))
                x_lim_max = max((numpy.nanmax(x), x_lim_max))
            else:
                x_lim_min = numpy.nanmin(x)
                x_lim_max = numpy.nanmax(x)
            
            if n2 == 1:
                #np_x
                for _x in x:
                    self.ax_pri_2.axvline(x=_x, linewidth=0.5, color="k", linestyle=":")
                #self.ax_pri_2.plot(x, 0*x, "k|", linewidth=1.0)
            elif n2 == 2:
                #np_xm
                y_mod = np_array[:, 1]
                if y_lim_min is not None:
                    y_lim_min = min((numpy.nanmin(y_mod), y_lim_min))
                    y_lim_max = max((numpy.nanmax(y_mod), y_lim_max))
                else:
                    y_lim_min = numpy.nanmin(y_mod)
                    y_lim_max = numpy.nanmax(y_mod)

                self.ax_pri_1.plot(x, y_mod, "k-", linewidth=1.0)
            elif n2 == 3:
                #np_xes
                y_exp = np_array[:, 1]
                y_sig = np_array[:, 2]
                if y_lim_min is not None:
                    y_lim_min = min((numpy.nanmin(y_exp-y_sig), y_lim_min))
                    y_lim_max = max((numpy.nanmax(y_exp+y_sig), y_lim_max))
                else:
                    y_lim_min = numpy.nanmin(y_exp)
                    y_lim_max = numpy.nanmax(y_exp)
                self.ax_pri_1.errorbar(x, y_exp, yerr = y_sig, ecolor = col_1,  fmt='.', color=col_1, linewidth = 0.5)
            elif n2 == 4:
                #np_xesm
                y_exp = np_array[:, 1]
                y_sig = np_array[:, 2]
                y_mod = np_array[:, 3]
                if y_lim_min is not None:
                    y_lim_min = min((numpy.nanmin(y_mod), numpy.nanmin(y_exp-y_sig), y_lim_min))
                    y_lim_max = max((numpy.nanmax(y_mod), numpy.nanmax(y_exp+y_sig), y_lim_max))
                else:
                    y_lim_min = min(numpy.nanmin(y_mod), numpy.nanmin(y_exp-y_sig))
                    y_lim_max = max(numpy.nanmax(y_mod), numpy.nanmax(y_exp+y_sig))

                y_lim_min_2 = min((numpy.nanmin(y_exp-y_mod), y_lim_min_2))
                y_lim_max_2 = max((numpy.nanmax(y_exp-y_mod), y_lim_max_2))


                self.ax_pri_1.errorbar(x, y_exp, yerr = y_sig, ecolor = col_1,  fmt='.', color=col_1, linewidth = 0.5)
                self.ax_pri_1.plot(x, y_mod, "k-", linewidth=1.0)
                self.ax_pri_2.plot(x, y_exp-y_mod, "k-", linewidth=1.0)

        x_diff = x_lim_max - x_lim_min
        y_diff = y_lim_max - y_lim_min
        y_diff_2 = y_lim_max_2 - y_lim_min_2
        self.xlim_orig = (x_lim_min-0.05*x_diff, x_lim_max+0.05*x_diff)
        self.ylim_orig_1 = (y_lim_min-0.05*y_diff, y_lim_max+0.05*y_diff)
        self.ylim_orig_2 = (y_lim_min_2-0.05*y_diff_2, y_lim_max_2+0.05*y_diff_2)
        
        if self.ax_pri_1 is not None: self.ax_pri_1.set_xlim(self.xlim_orig)
        if self.ax_pri_2 is not None: self.ax_pri_2.set_xlim(self.xlim_orig)
        if self.ax_pri_1 is not None: self.ax_pri_1.set_ylim(self.ylim_orig_1)
        if self.ax_pri_2 is not None: self.ax_pri_2.set_ylim(self.ylim_orig_2)


    def onclick(self, event):
        if event.button == 3:
            if self.info_press == (False, False):
                self.info_press = (True, False)
                self.xlim = [event.xdata]
                self.ylim = [event.ydata]
            elif self.info_press == (True, False):
                self.info_press = (True, True)
                self.xlim.append(event.xdata)
                self.ylim.append(event.ydata)
            if self.info_press == (True, True):
                self.info_press = (False, False)
                xlim = (min(self.xlim), max(self.xlim))
                ylim = (min(self.ylim), max(self.ylim))
                if self.ax_pri_1 is not None: self.ax_pri_1.set_xlim(xlim)
                if self.ax_pri_2 is not None: self.ax_pri_2.set_xlim(xlim)
                if self.ax_pri_1 is not None: self.ax_pri_1.set_ylim(ylim)
                self.xlim = []
                self.ylim = []
                self.draw()
        elif event.button == 1:
                self.info_press == (False, False)
                if self.ax_pri_1 is not None:  self.ax_pri_1.set_xlim(self.xlim_orig)
                if self.ax_pri_2 is not None: self.ax_pri_2.set_xlim(self.xlim_orig)
                if self.ax_pri_1 is not None: self.ax_pri_1.set_ylim(self.ylim_orig_1)
                if self.ax_pri_2 is not None: self.ax_pri_2.set_ylim(self.ylim_orig_2)
                self.xlim = []
                self.ylim = []
                self.draw()
        else:
            self.info_press == (False, False)
            self.xlim = []
            self.ylim = []
            
 
if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    fname = "NaCaAlF_exp.out"
    ex = cwind_central()
    
    sys.exit(app.exec_())



import numpy
from PyQt5 import QtWidgets, QtGui, QtCore
from .interactive_graph_mod_mono import cwidg_central

from .FUNCTIONS import make_qtablewidget_for_loop_constr, show_info, del_layout
from .w_loop_constr import w_for_loop_constr

def w_for_meas_inelasticl(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    w_for_loop_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread)

    if obj.count_calc is not None:
        del_layout(layout_3)
        widg_graph_1d = cwidg_central()
        widg_graph_1d.plot_lines(obj.energy, l_y=[obj.count, obj.count_calc], l_y_sig=[obj.count_sigma, None])
        layout_3.addWidget(widg_graph_1d)
    
    return 



from .FUNCTIONS import make_qtablewidget_for_loop_constr, show_info, show_variables, show_help, \
    get_layout_method_help, del_layout, show_widget, get_layout_rciftab_obj, \
    get_layout_internal_attribute
from .w_item_constr import w_for_item_constr
from PyQt5 import QtWidgets, QtGui, QtCore


def w_for_loop_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    layout_11.addWidget(QtWidgets.QLabel("Loop name:"))

    _l_e_dist = QtWidgets.QLineEdit()
    _l_e_dist.setText(obj.loop_name)
    _l_e_dist.editingFinished.connect(lambda: setattr(obj, "loop_name", _l_e_dist.text()))
    layout_11.addWidget(_l_e_dist)

    w_t = make_qtablewidget_for_loop_constr(obj, label=w_output)
    layout_11.addWidget(w_t, 2)

    lay_left_11 = QtWidgets.QHBoxLayout()
    _b_add = QtWidgets.QPushButton("add/delete item")
    lay_left_11.addWidget(_b_add)

    _b_show_item = QtWidgets.QPushButton("show item")
    lay_left_11.addWidget(_b_show_item)

    _b_info = QtWidgets.QPushButton("info")
    _b_info.clicked.connect(lambda: show_info(obj, w_output))
    lay_left_11.addWidget(_b_info)
    layout_11.addLayout(lay_left_11)

    lay_left_2 = get_layout_internal_attribute(obj, w_output)
    layout_12.addLayout(lay_left_2)

    lay_left_3 = get_layout_method_help(obj, w_output, thread)
    layout_13.addLayout(lay_left_3)

    _lay_3 = get_layout_rciftab_obj(obj, thread)
    layout_2.addLayout(_lay_3)

    _b_show_item.clicked.connect(lambda: show_item_in_loop(obj, w_t.selectedRanges(), layout_3, w_output, thread))
    return


"""
def plot_graph(qtablewidget, widg_graph_1d):
    selected_items = qtablewidget.selectedItems()
    print("selected_items")
    l_col = [_h.column() for _h in selected_items]
    if len(l_col) == 0:
        return
    columns = set(l_col)
    column_min = min(columns)
    columns_y = sorted(columns-set([column_min]))

    try:
        x = [float(_h.text()) for _h in selected_items if _h.column()==column_min]
    except:
        return None
    l_y = []
    for _column in columns_y:
        try:
            y = [float(_h.text()) for _h in selected_items if _h.column()==_column]
            l_y.append(y)
        except:
            pass
    xy_s =  sorted(zip(x, *l_y), key=lambda _: _[0])
    
    x = list(zip(*xy_s))[0]
    if len(l_y) >= 1:
        l_y = list(zip(*xy_s))[1:]
        widg_graph_1d.plot_lines(x, l_y=l_y)
    else:
        widg_graph_1d.plot_lines(range(len(x)), l_y==[x])
"""


def add_del_item_in_loop(widg, obj):
    ls_out = ["Enter the number of items:"]

    text, ok = QtWidgets.QInputDialog.getText(widg, 'Input Dialog',
                                              "\n".join(ls_out))
    if not (ok):
        return None

    _ind = int(text)
    l_item = obj.item
    if len(l_item) <= _ind:
        _item_class = obj.ITEM_CLASS
        for _i in range(_ind - len(l_item)):
            l_item.append(_item_class())
    elif len(l_item) > _ind:
        l_item_new = l_item[:(_ind + 1)]
        obj.item = l_item_new


def show_item_in_loop(obj, selected_ranges, layout, w_output, thread):
    del_layout(layout)
    row = None
    item = None
    if len(selected_ranges) >= 1:
        row = selected_ranges[0].topRow()
    if row is not None:
        if len(obj.item) > row:
            item = obj.item[row]
    if item is not None:

        l_11 = QtWidgets.QVBoxLayout()
        l_12 = QtWidgets.QVBoxLayout()
        l_13 = QtWidgets.QVBoxLayout()
        l_2 = QtWidgets.QVBoxLayout()
        l_3 = QtWidgets.QVBoxLayout()
        w_for_item_constr(item, l_11, l_12, l_13, l_2, l_3, w_output, thread)
        lay_main = QtWidgets.QHBoxLayout()
        lay_main.addLayout(l_11)
        lay_main.addLayout(l_12)
        lay_main.addLayout(l_13)

        layout.addLayout(lay_main)



import numpy
from PyQt5 import QtWidgets, QtGui, QtCore
from .i_graph_mod_1d import cwidg_central as cwidg_pwd
from .FUNCTIONS import get_layout_rciftab_obj, del_layout
from .w_loop_constr import w_for_loop_constr


def w_for_diffrn_refln(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    w_for_loop_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread)
    del_layout(layout_3)

    np_x_1 = numpy.array(obj.fr_calc, dtype = float)
    np_y_1 = numpy.array(obj.fr, dtype = float)
    np_s_1 = numpy.array(obj.fr_sigma, dtype = float)
    if (numpy.all(numpy.isnan(np_x_1)) | numpy.all(numpy.isnan(np_y_1)) | numpy.all(numpy.isnan(np_s_1))):
        return
    np_xys_1 = numpy.vstack((np_x_1, np_y_1, np_s_1)).transpose()
    np_xm_1 = numpy.vstack((np_x_1, np_x_1)).transpose()

    w_s_1 = cwidg_pwd()
    w_s_1.plot_numpy_arrays(np_xys_1, np_xm_1)

    layout_3.addWidget(w_s_1)
    return



import numpy
from PyQt5 import QtWidgets, QtGui, QtCore
from .interactive_graph_mod_mono import cwidg_central

from .FUNCTIONS import make_qtablewidget_for_loop_constr, show_info


def widget_for_meas_inelasticl(obj, label_out):
    lay_to_fill = QtWidgets.QHBoxLayout()

    lay_left = QtWidgets.QVBoxLayout()

    
    w_t = make_qtablewidget_for_loop_constr(obj)
        
    lay_left.addWidget(w_t)

    if obj.count_calc is not None:
        widg_graph_1d = cwidg_central()
        widg_graph_1d.plot_lines(obj.energy, l_y=[obj.count, obj.count_calc], l_y_sig=[obj.count_sigma, None])
        lay_left.addWidget(widg_graph_1d)

    _b_info = QtWidgets.QPushButton("info")
    _b_info.clicked.connect(lambda : show_info(obj, label_out))
    lay_left.addWidget(_b_info)


    lay_to_fill.addLayout(lay_left)
    widg_out = QtWidgets.QWidget()
    widg_out.setLayout(lay_to_fill)
    
    return widg_out



import numpy
from PyQt5 import QtWidgets, QtGui, QtCore
from .FUNCTIONS import show_info

def widget_for_meas_elasticl(obj):
    lay_to_fill = QtWidgets.QHBoxLayout()

    lay_right = QtWidgets.QVBoxLayout()
    size_pol_2 =  QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
    label_out = QtWidgets.QLabel()
    label_out.setSizePolicy(size_pol_2)
    label_out.setFont(QtGui.QFont("Courier", 8, QtGui.QFont.Normal))
    label_out.setTextInteractionFlags(QtCore.Qt.TextSelectableByMouse)
    area = QtWidgets.QScrollArea()
    area.setWidgetResizable(True)
    area.setWidget(label_out)
    lay_right.addWidget(area)

    lay_left = QtWidgets.QVBoxLayout()

    n_row, n_col = len(obj.moment_11), 1
    l_name = ["moment_11"]
    _i_row = 0
    l_add = n_row*[None]
    if obj.moment_11_calc is not None:
        if len(obj.moment_11_calc) == n_row:
            l_add  = obj.moment_11_calc
            n_col = 2
            l_name = ["moment_11", "moment_11_calc"]
    w_t = QtWidgets.QTableWidget(n_row, n_col)
    w_t.setHorizontalHeaderLabels(l_name)
        
    for _0, _1 in zip(obj.moment_11, l_add): 
        _w_ti_0 = QtWidgets.QTableWidgetItem()
        _w_ti_0.setText(str(_0))
        w_t.setItem(_i_row, 0, _w_ti_0)
        if _1 is not None:
            _w_ti_1 = QtWidgets.QTableWidgetItem()
            _w_ti_1.setText(f"{_1:.5f}")
            w_t.setItem(_i_row, 1, _w_ti_1)
        _i_row += 1
    lay_left.addWidget(w_t)



    _b_info = QtWidgets.QPushButton("info")
    _b_info.clicked.connect(lambda : show_info(obj, label_out))
    lay_left.addWidget(_b_info)


    lay_to_fill.addLayout(lay_left)
    lay_to_fill.addLayout(lay_right)
    widg_out = QtWidgets.QWidget()
    widg_out.setLayout(lay_to_fill)
    
    return widg_out



from .FUNCTIONS import make_qtablewidget_for_item_constr, show_info, get_layout_method_help, \
    get_layout_internal_attribute, get_layout_rciftab_obj

from PyQt5 import QtWidgets, QtGui, QtCore


def w_for_item_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):
    layout_11.addWidget(QtWidgets.QLabel("Mandatory and optional attributes"))
    w_t = make_qtablewidget_for_item_constr(obj, label=w_output)
    layout_11.addWidget(w_t)

    _b_info = QtWidgets.QPushButton("info")
    _b_info.clicked.connect(lambda: show_info(obj, w_output))
    layout_11.addWidget(_b_info)

    lay_left_2 = get_layout_internal_attribute(obj, w_output)
    layout_12.addLayout(lay_left_2)

    lay_left_3 = get_layout_method_help(obj, w_output, thread)
    layout_13.addLayout(lay_left_3)


    _lay_3 = get_layout_rciftab_obj(obj, thread)
    layout_2.addLayout(_lay_3)
    return



import os
import numpy
from PyQt5 import QtWidgets, QtGui, QtCore

from cryspy_editor.b_rcif_to_cryspy import L_ITEM_CLASS, L_LOOP_CLASS, L_DATA_CLASS
from .FUNCTIONS import show_info, get_layout_method_help, add_mandatory_optional_obj, make_qtablewidget_for_data_constr, show_widget

def w_for_global_constr(obj, layout_11, layout_12, layout_13, layout_2, layout_3, w_output, thread):

    layout_11.addWidget(QtWidgets.QLabel("Global name:"))
    _l_e_dist = QtWidgets.QLineEdit()
    _l_e_dist.setText(obj.global_name)
    _l_e_dist.editingFinished.connect(lambda : setattr(obj, "global_name", _l_e_dist.text()))
    layout_11.addWidget(_l_e_dist)

    
    layout_11.addWidget(QtWidgets.QLabel("Defined attributes:"))
    _widg_table = make_qtablewidget_for_data_constr(obj, w_output, thread)
    layout_11.addWidget(_widg_table)


    _b_info = QtWidgets.QPushButton("info")
    _b_info.clicked.connect(lambda : show_info(obj, w_output))
    layout_11.addWidget(_b_info)

    layout_12.addLayout(add_mandatory_optional_obj(obj, w_output, thread))

    layout_13.addLayout(get_layout_method_help(obj, w_output, thread))

    _text_edit = QtWidgets.QTextEdit()
    _text_edit.setText(obj.to_cif())
    layout_2.addWidget(_text_edit)

    return

